"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice_py import errors, models, utils
from flexprice_py._hooks import HookContext
from flexprice_py.types import OptionalNullable, UNSET
from flexprice_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class Entitlements(BaseSDK):
    def get_addon_entitlements(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListEntitlementsResponse:
        r"""Get addon entitlements

        Use when checking what features or limits an addon grants (e.g. for display or entitlement logic).

        :param id: Addon ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetAddonEntitlementsRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/addons/{id}/entitlements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getAddonEntitlements",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListEntitlementsResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_addon_entitlements_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListEntitlementsResponse:
        r"""Get addon entitlements

        Use when checking what features or limits an addon grants (e.g. for display or entitlement logic).

        :param id: Addon ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetAddonEntitlementsRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/addons/{id}/entitlements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getAddonEntitlements",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListEntitlementsResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def create_entitlement(
        self,
        *,
        feature_id: str,
        feature_type: models.components.FeatureType,
        end_date: Optional[str] = None,
        entity_id: Optional[str] = None,
        entity_type: Optional[models.components.EntitlementEntityType] = None,
        is_enabled: Optional[bool] = None,
        is_soft_limit: Optional[bool] = None,
        parent_entitlement_id: Optional[str] = None,
        plan_id: Optional[str] = None,
        start_date: Optional[str] = None,
        static_value: Optional[str] = None,
        usage_limit: Optional[int] = None,
        usage_reset_period: Optional[
            models.components.EntitlementUsageResetPeriod
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoEntitlementResponse:
        r"""Create entitlement

        Use when attaching a feature (and its limit) to a plan or addon (e.g. \"10 seats\" or \"1000 API calls\"). Defines what the plan/addon includes.

        :param feature_id:
        :param feature_type:
        :param end_date:
        :param entity_id:
        :param entity_type:
        :param is_enabled:
        :param is_soft_limit:
        :param parent_entitlement_id:
        :param plan_id:
        :param start_date:
        :param static_value:
        :param usage_limit:
        :param usage_reset_period:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateEntitlementRequest(
            end_date=end_date,
            entity_id=entity_id,
            entity_type=entity_type,
            feature_id=feature_id,
            feature_type=feature_type,
            is_enabled=is_enabled,
            is_soft_limit=is_soft_limit,
            parent_entitlement_id=parent_entitlement_id,
            plan_id=plan_id,
            start_date=start_date,
            static_value=static_value,
            usage_limit=usage_limit,
            usage_reset_period=usage_reset_period,
        )

        req = self._build_request(
            method="POST",
            path="/entitlements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoCreateEntitlementRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createEntitlement",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoEntitlementResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def create_entitlement_async(
        self,
        *,
        feature_id: str,
        feature_type: models.components.FeatureType,
        end_date: Optional[str] = None,
        entity_id: Optional[str] = None,
        entity_type: Optional[models.components.EntitlementEntityType] = None,
        is_enabled: Optional[bool] = None,
        is_soft_limit: Optional[bool] = None,
        parent_entitlement_id: Optional[str] = None,
        plan_id: Optional[str] = None,
        start_date: Optional[str] = None,
        static_value: Optional[str] = None,
        usage_limit: Optional[int] = None,
        usage_reset_period: Optional[
            models.components.EntitlementUsageResetPeriod
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoEntitlementResponse:
        r"""Create entitlement

        Use when attaching a feature (and its limit) to a plan or addon (e.g. \"10 seats\" or \"1000 API calls\"). Defines what the plan/addon includes.

        :param feature_id:
        :param feature_type:
        :param end_date:
        :param entity_id:
        :param entity_type:
        :param is_enabled:
        :param is_soft_limit:
        :param parent_entitlement_id:
        :param plan_id:
        :param start_date:
        :param static_value:
        :param usage_limit:
        :param usage_reset_period:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateEntitlementRequest(
            end_date=end_date,
            entity_id=entity_id,
            entity_type=entity_type,
            feature_id=feature_id,
            feature_type=feature_type,
            is_enabled=is_enabled,
            is_soft_limit=is_soft_limit,
            parent_entitlement_id=parent_entitlement_id,
            plan_id=plan_id,
            start_date=start_date,
            static_value=static_value,
            usage_limit=usage_limit,
            usage_reset_period=usage_reset_period,
        )

        req = self._build_request_async(
            method="POST",
            path="/entitlements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoCreateEntitlementRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createEntitlement",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoEntitlementResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def create_entitlements_bulk(
        self,
        *,
        items: Union[
            List[models.components.DtoCreateEntitlementRequest],
            List[models.components.DtoCreateEntitlementRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCreateBulkEntitlementResponse:
        r"""Create entitlements in bulk

        Use when attaching many features to a plan or addon at once (e.g. initial plan setup or import). Bulk version of create entitlement.

        :param items:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateBulkEntitlementRequest(
            items=utils.get_pydantic_model(
                items, List[models.components.DtoCreateEntitlementRequest]
            ),
        )

        req = self._build_request(
            method="POST",
            path="/entitlements/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoCreateBulkEntitlementRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createEntitlementsBulk",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCreateBulkEntitlementResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def create_entitlements_bulk_async(
        self,
        *,
        items: Union[
            List[models.components.DtoCreateEntitlementRequest],
            List[models.components.DtoCreateEntitlementRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCreateBulkEntitlementResponse:
        r"""Create entitlements in bulk

        Use when attaching many features to a plan or addon at once (e.g. initial plan setup or import). Bulk version of create entitlement.

        :param items:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateBulkEntitlementRequest(
            items=utils.get_pydantic_model(
                items, List[models.components.DtoCreateEntitlementRequest]
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/entitlements/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoCreateBulkEntitlementRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createEntitlementsBulk",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCreateBulkEntitlementResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def query_entitlement(
        self,
        *,
        end_time: Optional[str] = None,
        entity_ids: Optional[List[str]] = None,
        entity_type: Optional[models.components.EntitlementEntityType] = None,
        expand: Optional[str] = None,
        feature_ids: Optional[List[str]] = None,
        feature_type: Optional[models.components.FeatureType] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        is_enabled: Optional[bool] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.EntitlementFilterOrder] = None,
        plan_ids: Optional[List[str]] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        start_time: Optional[str] = None,
        status: Optional[models.components.Status] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListEntitlementsResponse:
        r"""Query entitlements

        Use when listing or searching entitlements (e.g. plan editor or audit). Returns a paginated list; supports filtering by plan, addon, feature.

        :param end_time:
        :param entity_ids:
        :param entity_type:
        :param expand:
        :param feature_ids:
        :param feature_type:
        :param filters: Specific filters for entitlements
        :param is_enabled:
        :param limit:
        :param offset:
        :param order:
        :param plan_ids:
        :param sort:
        :param start_time:
        :param status:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.EntitlementFilter(
            end_time=end_time,
            entity_ids=entity_ids,
            entity_type=entity_type,
            expand=expand,
            feature_ids=feature_ids,
            feature_type=feature_type,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            is_enabled=is_enabled,
            limit=limit,
            offset=offset,
            order=order,
            plan_ids=plan_ids,
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            start_time=start_time,
            status=status,
        )

        req = self._build_request(
            method="POST",
            path="/entitlements/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.EntitlementFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryEntitlement",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListEntitlementsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def query_entitlement_async(
        self,
        *,
        end_time: Optional[str] = None,
        entity_ids: Optional[List[str]] = None,
        entity_type: Optional[models.components.EntitlementEntityType] = None,
        expand: Optional[str] = None,
        feature_ids: Optional[List[str]] = None,
        feature_type: Optional[models.components.FeatureType] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        is_enabled: Optional[bool] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.EntitlementFilterOrder] = None,
        plan_ids: Optional[List[str]] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        start_time: Optional[str] = None,
        status: Optional[models.components.Status] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListEntitlementsResponse:
        r"""Query entitlements

        Use when listing or searching entitlements (e.g. plan editor or audit). Returns a paginated list; supports filtering by plan, addon, feature.

        :param end_time:
        :param entity_ids:
        :param entity_type:
        :param expand:
        :param feature_ids:
        :param feature_type:
        :param filters: Specific filters for entitlements
        :param is_enabled:
        :param limit:
        :param offset:
        :param order:
        :param plan_ids:
        :param sort:
        :param start_time:
        :param status:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.EntitlementFilter(
            end_time=end_time,
            entity_ids=entity_ids,
            entity_type=entity_type,
            expand=expand,
            feature_ids=feature_ids,
            feature_type=feature_type,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            is_enabled=is_enabled,
            limit=limit,
            offset=offset,
            order=order,
            plan_ids=plan_ids,
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            start_time=start_time,
            status=status,
        )

        req = self._build_request_async(
            method="POST",
            path="/entitlements/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.EntitlementFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryEntitlement",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListEntitlementsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_entitlement(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoEntitlementResponse:
        r"""Get entitlement

        Use when you need to load a single entitlement (e.g. to display or edit a feature limit).

        :param id: Entitlement ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetEntitlementRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/entitlements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getEntitlement",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoEntitlementResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_entitlement_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoEntitlementResponse:
        r"""Get entitlement

        Use when you need to load a single entitlement (e.g. to display or edit a feature limit).

        :param id: Entitlement ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetEntitlementRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/entitlements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getEntitlement",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoEntitlementResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def update_entitlement(
        self,
        *,
        id: str,
        is_enabled: Optional[bool] = None,
        is_soft_limit: Optional[bool] = None,
        static_value: Optional[str] = None,
        usage_limit: Optional[int] = None,
        usage_reset_period: Optional[
            models.components.EntitlementUsageResetPeriod
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoEntitlementResponse:
        r"""Update entitlement

        Use when changing an entitlement (e.g. increasing or decreasing a limit). Request body contains the fields to update.

        :param id: Entitlement ID
        :param is_enabled:
        :param is_soft_limit:
        :param static_value:
        :param usage_limit:
        :param usage_reset_period:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdateEntitlementRequest(
            id=id,
            body=models.components.DtoUpdateEntitlementRequest(
                is_enabled=is_enabled,
                is_soft_limit=is_soft_limit,
                static_value=static_value,
                usage_limit=usage_limit,
                usage_reset_period=usage_reset_period,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/entitlements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdateEntitlementRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateEntitlement",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoEntitlementResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def update_entitlement_async(
        self,
        *,
        id: str,
        is_enabled: Optional[bool] = None,
        is_soft_limit: Optional[bool] = None,
        static_value: Optional[str] = None,
        usage_limit: Optional[int] = None,
        usage_reset_period: Optional[
            models.components.EntitlementUsageResetPeriod
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoEntitlementResponse:
        r"""Update entitlement

        Use when changing an entitlement (e.g. increasing or decreasing a limit). Request body contains the fields to update.

        :param id: Entitlement ID
        :param is_enabled:
        :param is_soft_limit:
        :param static_value:
        :param usage_limit:
        :param usage_reset_period:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdateEntitlementRequest(
            id=id,
            body=models.components.DtoUpdateEntitlementRequest(
                is_enabled=is_enabled,
                is_soft_limit=is_soft_limit,
                static_value=static_value,
                usage_limit=usage_limit,
                usage_reset_period=usage_reset_period,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/entitlements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdateEntitlementRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateEntitlement",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoEntitlementResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def delete_entitlement(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoSuccessResponse:
        r"""Delete entitlement

        Use when removing a feature from a plan or addon (e.g. deprecating a capability). Returns 200 with success message.

        :param id: Entitlement ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.DeleteEntitlementRequest(
            id=id,
        )

        req = self._build_request(
            method="DELETE",
            path="/entitlements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteEntitlement",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoSuccessResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def delete_entitlement_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoSuccessResponse:
        r"""Delete entitlement

        Use when removing a feature from a plan or addon (e.g. deprecating a capability). Returns 200 with success message.

        :param id: Entitlement ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.DeleteEntitlementRequest(
            id=id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/entitlements/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteEntitlement",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoSuccessResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_plan_entitlements(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPlanResponse:
        r"""Get plan entitlements

        Use when checking what a plan includes (e.g. feature list or limits for display or gating).

        :param id: Plan ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetPlanEntitlementsRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/plans/{id}/entitlements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPlanEntitlements",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.components.DtoPlanResponse, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_plan_entitlements_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPlanResponse:
        r"""Get plan entitlements

        Use when checking what a plan includes (e.g. feature list or limits for display or gating).

        :param id: Plan ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetPlanEntitlementsRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/plans/{id}/entitlements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPlanEntitlements",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.components.DtoPlanResponse, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cancelimmediatelyinvoicepolicy import CancelImmediatelyInvoicePolicy
from .cancellationtype import CancellationType
from .prorationbehavior import ProrationBehavior
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoCancelSubscriptionRequestTypedDict(TypedDict):
    cancellation_type: CancellationType
    cancel_immediately_inovice_policy: NotRequired[CancelImmediatelyInvoicePolicy]
    proration_behavior: NotRequired[ProrationBehavior]
    reason: NotRequired[str]
    r"""Reason for cancellation (for audit and business intelligence)"""


class DtoCancelSubscriptionRequest(BaseModel):
    cancellation_type: CancellationType

    cancel_immediately_inovice_policy: Optional[CancelImmediatelyInvoicePolicy] = None

    proration_behavior: Optional[ProrationBehavior] = None

    reason: Optional[str] = None
    r"""Reason for cancellation (for audit and business intelligence)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["cancel_immediately_inovice_policy", "proration_behavior", "reason"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


DataType = Literal[
    "string",
    "number",
    "date",
    "array",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cancellationtype import CancellationType
from .dto_invoiceresponse import DtoInvoiceResponse, DtoInvoiceResponseTypedDict
from .dto_prorationdetail import DtoProrationDetail, DtoProrationDetailTypedDict
from .subscriptionstatus import SubscriptionStatus
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCancelSubscriptionResponseTypedDict(TypedDict):
    cancellation_type: NotRequired[CancellationType]
    effective_date: NotRequired[str]
    message: NotRequired[str]
    r"""Response metadata"""
    processed_at: NotRequired[str]
    proration_details: NotRequired[List[DtoProrationDetailTypedDict]]
    proration_invoice: NotRequired[DtoInvoiceResponseTypedDict]
    reason: NotRequired[str]
    status: NotRequired[SubscriptionStatus]
    subscription_id: NotRequired[str]
    r"""Basic cancellation info"""
    total_credit_amount: NotRequired[str]


class DtoCancelSubscriptionResponse(BaseModel):
    cancellation_type: Optional[CancellationType] = None

    effective_date: Optional[str] = None

    message: Optional[str] = None
    r"""Response metadata"""

    processed_at: Optional[str] = None

    proration_details: Optional[List[DtoProrationDetail]] = None

    proration_invoice: Optional[DtoInvoiceResponse] = None

    reason: Optional[str] = None

    status: Optional[SubscriptionStatus] = None

    subscription_id: Optional[str] = None
    r"""Basic cancellation info"""

    total_credit_amount: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "cancellation_type",
                "effective_date",
                "message",
                "processed_at",
                "proration_details",
                "proration_invoice",
                "reason",
                "status",
                "subscription_id",
                "total_credit_amount",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import Any, TYPE_CHECKING

from flexprice_py.utils.dynamic_imports import lazy_getattr, lazy_dir

if TYPE_CHECKING:
    from .activatesubscription import (
        ActivateSubscriptionRequest,
        ActivateSubscriptionRequestTypedDict,
    )
    from .attemptinvoicepayment import (
        AttemptInvoicePaymentRequest,
        AttemptInvoicePaymentRequestTypedDict,
    )
    from .cancelsubscription import (
        CancelSubscriptionRequest,
        CancelSubscriptionRequestTypedDict,
    )
    from .cancelsubscriptionschedule import (
        CancelSubscriptionScheduleRequest,
        CancelSubscriptionScheduleRequestTypedDict,
    )
    from .createorupdateintegration import (
        CreateOrUpdateIntegrationRequest,
        CreateOrUpdateIntegrationRequestTypedDict,
    )
    from .createsubscriptionlineitem import (
        CreateSubscriptionLineItemRequest,
        CreateSubscriptionLineItemRequestTypedDict,
    )
    from .deleteaddon import DeleteAddonRequest, DeleteAddonRequestTypedDict
    from .deleteapikey import DeleteAPIKeyRequest, DeleteAPIKeyRequestTypedDict
    from .deletecostsheet import DeleteCostsheetRequest, DeleteCostsheetRequestTypedDict
    from .deletecoupon import DeleteCouponRequest, DeleteCouponRequestTypedDict
    from .deletecreditgrant import (
        DeleteCreditGrantRequest,
        DeleteCreditGrantRequestTypedDict,
    )
    from .deletecustomer import DeleteCustomerRequest, DeleteCustomerRequestTypedDict
    from .deleteentitlement import (
        DeleteEntitlementRequest,
        DeleteEntitlementRequestTypedDict,
    )
    from .deleteentityintegrationmapping import (
        DeleteEntityIntegrationMappingRequest,
        DeleteEntityIntegrationMappingRequestTypedDict,
    )
    from .deletefeature import DeleteFeatureRequest, DeleteFeatureRequestTypedDict
    from .deletegroup import DeleteGroupRequest, DeleteGroupRequestTypedDict
    from .deleteintegration import (
        DeleteIntegrationRequest,
        DeleteIntegrationRequestTypedDict,
    )
    from .deletepayment import DeletePaymentRequest, DeletePaymentRequestTypedDict
    from .deleteplan import DeletePlanRequest, DeletePlanRequestTypedDict
    from .deleteprice import DeletePriceRequest, DeletePriceRequestTypedDict
    from .deletepriceunit import DeletePriceUnitRequest, DeletePriceUnitRequestTypedDict
    from .deletescheduledtask import (
        DeleteScheduledTaskRequest,
        DeleteScheduledTaskRequestTypedDict,
    )
    from .deletesubscriptionlineitem import (
        DeleteSubscriptionLineItemRequest,
        DeleteSubscriptionLineItemRequestTypedDict,
    )
    from .deletetaxassociation import (
        DeleteTaxAssociationRequest,
        DeleteTaxAssociationRequestTypedDict,
    )
    from .deletetaxrate import DeleteTaxRateRequest, DeleteTaxRateRequestTypedDict
    from .downloadtaskexport import (
        DownloadTaskExportRequest,
        DownloadTaskExportRequestTypedDict,
    )
    from .executesubscriptionchange import (
        ExecuteSubscriptionChangeRequest,
        ExecuteSubscriptionChangeRequestTypedDict,
    )
    from .finalizeinvoice import FinalizeInvoiceRequest, FinalizeInvoiceRequestTypedDict
    from .getaddon import GetAddonRequest, GetAddonRequestTypedDict
    from .getaddonbylookupkey import (
        GetAddonByLookupKeyRequest,
        GetAddonByLookupKeyRequestTypedDict,
    )
    from .getaddonentitlements import (
        GetAddonEntitlementsRequest,
        GetAddonEntitlementsRequestTypedDict,
    )
    from .getcostsheet import GetCostsheetRequest, GetCostsheetRequestTypedDict
    from .getcoupon import GetCouponRequest, GetCouponRequestTypedDict
    from .getcreditgrant import GetCreditGrantRequest, GetCreditGrantRequestTypedDict
    from .getcreditnote import GetCreditNoteRequest, GetCreditNoteRequestTypedDict
    from .getcustomer import GetCustomerRequest, GetCustomerRequestTypedDict
    from .getcustomerbyexternalid import (
        GetCustomerByExternalIDRequest,
        GetCustomerByExternalIDRequestTypedDict,
    )
    from .getcustomerentitlements import (
        GetCustomerEntitlementsRequest,
        GetCustomerEntitlementsRequestTypedDict,
    )
    from .getcustomerinvoicesummary import (
        GetCustomerInvoiceSummaryRequest,
        GetCustomerInvoiceSummaryRequestTypedDict,
    )
    from .getcustomerupcominggrants import (
        GetCustomerUpcomingGrantsRequest,
        GetCustomerUpcomingGrantsRequestTypedDict,
    )
    from .getcustomerusagesummary import (
        GetCustomerUsageSummaryRequest,
        GetCustomerUsageSummaryRequestTypedDict,
    )
    from .getcustomerwallets import (
        GetCustomerWalletsRequest,
        GetCustomerWalletsRequestTypedDict,
    )
    from .getentitlement import GetEntitlementRequest, GetEntitlementRequestTypedDict
    from .getevent import GetEventRequest, GetEventRequestTypedDict
    from .getgroup import GetGroupRequest, GetGroupRequestTypedDict
    from .getintegration import GetIntegrationRequest, GetIntegrationRequestTypedDict
    from .getinvoice import GetInvoiceRequest, GetInvoiceRequestTypedDict
    from .getinvoicepdf import GetInvoicePdfRequest, GetInvoicePdfRequestTypedDict
    from .getpayment import GetPaymentRequest, GetPaymentRequestTypedDict
    from .getplan import GetPlanRequest, GetPlanRequestTypedDict
    from .getplancreditgrants import (
        GetPlanCreditGrantsRequest,
        GetPlanCreditGrantsRequestTypedDict,
    )
    from .getplanentitlements import (
        GetPlanEntitlementsRequest,
        GetPlanEntitlementsRequestTypedDict,
    )
    from .getprice import GetPriceRequest, GetPriceRequestTypedDict
    from .getpricebylookupkey import (
        GetPriceByLookupKeyRequest,
        GetPriceByLookupKeyRequestTypedDict,
    )
    from .getpriceunit import GetPriceUnitRequest, GetPriceUnitRequestTypedDict
    from .getpriceunitbycode import (
        GetPriceUnitByCodeRequest,
        GetPriceUnitByCodeRequestTypedDict,
    )
    from .getrbacrole import GetRbacRoleRequest, GetRbacRoleRequestTypedDict
    from .getscheduledtask import (
        GetScheduledTaskRequest,
        GetScheduledTaskRequestTypedDict,
    )
    from .getsubscription import GetSubscriptionRequest, GetSubscriptionRequestTypedDict
    from .getsubscriptionaddonassociations import (
        GetSubscriptionAddonAssociationsRequest,
        GetSubscriptionAddonAssociationsRequestTypedDict,
    )
    from .getsubscriptionentitlements import (
        GetSubscriptionEntitlementsRequest,
        GetSubscriptionEntitlementsRequestTypedDict,
    )
    from .getsubscriptionschedule import (
        GetSubscriptionScheduleRequest,
        GetSubscriptionScheduleRequestTypedDict,
    )
    from .getsubscriptionupcominggrants import (
        GetSubscriptionUpcomingGrantsRequest,
        GetSubscriptionUpcomingGrantsRequestTypedDict,
    )
    from .getsubscriptionv2 import (
        GetSubscriptionV2Request,
        GetSubscriptionV2RequestTypedDict,
    )
    from .gettask import GetTaskRequest, GetTaskRequestTypedDict
    from .gettaskresult import GetTaskResultRequest, GetTaskResultRequestTypedDict
    from .gettaxassociation import (
        GetTaxAssociationRequest,
        GetTaxAssociationRequestTypedDict,
    )
    from .gettaxrate import GetTaxRateRequest, GetTaxRateRequestTypedDict
    from .gettaxrates import (
        GetTaxRatesOrder,
        GetTaxRatesRequest,
        GetTaxRatesRequestTypedDict,
        GetTaxRatesStatus,
        Scope,
    )
    from .gettenantbyid import GetTenantByIDRequest, GetTenantByIDRequestTypedDict
    from .getwallet import GetWalletRequest, GetWalletRequestTypedDict
    from .getwalletbalance import (
        GetWalletBalanceRequest,
        GetWalletBalanceRequestTypedDict,
    )
    from .getwalletsbycustomerid import (
        GetWalletsByCustomerIDRequest,
        GetWalletsByCustomerIDRequestTypedDict,
    )
    from .getwallettransactions import (
        GetWalletTransactionsOrder,
        GetWalletTransactionsRequest,
        GetWalletTransactionsRequestTypedDict,
        GetWalletTransactionsStatus,
        TransactionReason,
        TransactionStatus,
        Type,
    )
    from .handlechargebeewebhook import (
        HandleChargebeeWebhookRequest,
        HandleChargebeeWebhookRequestTypedDict,
    )
    from .handlehubspotwebhook import (
        HandleHubspotWebhookRequest,
        HandleHubspotWebhookRequestTypedDict,
    )
    from .handlemoyasarwebhook import (
        HandleMoyasarWebhookRequest,
        HandleMoyasarWebhookRequestTypedDict,
    )
    from .handlenomodwebhook import (
        HandleNomodWebhookRequest,
        HandleNomodWebhookRequestTypedDict,
    )
    from .handlequickbookswebhook import (
        HandleQuickbooksWebhookRequest,
        HandleQuickbooksWebhookRequestTypedDict,
    )
    from .handlerazorpaywebhook import (
        HandleRazorpayWebhookRequest,
        HandleRazorpayWebhookRequestTypedDict,
    )
    from .handlestripewebhook import (
        HandleStripeWebhookRequest,
        HandleStripeWebhookRequestTypedDict,
    )
    from .listallsubscriptionschedules import (
        ListAllSubscriptionSchedulesRequest,
        ListAllSubscriptionSchedulesRequestTypedDict,
    )
    from .listapikeys import ListAPIKeysRequest, ListAPIKeysRequestTypedDict
    from .listpayments import (
        ListPaymentsOrder,
        ListPaymentsRequest,
        ListPaymentsRequestTypedDict,
        ListPaymentsStatus,
    )
    from .listpriceunits import (
        ListPriceUnitsOrder,
        ListPriceUnitsRequest,
        ListPriceUnitsRequestTypedDict,
    )
    from .listscheduledtasks import (
        ListScheduledTasksRequest,
        ListScheduledTasksRequestTypedDict,
    )
    from .listsubscriptionpauses import (
        ListSubscriptionPausesRequest,
        ListSubscriptionPausesRequestTypedDict,
    )
    from .listsubscriptionschedules import (
        ListSubscriptionSchedulesRequest,
        ListSubscriptionSchedulesRequestTypedDict,
    )
    from .listtasks import (
        EntityType,
        ListTasksOrder,
        ListTasksRequest,
        ListTasksRequestTypedDict,
        ListTasksStatus,
        TaskStatus,
        TaskType,
    )
    from .listtaxassociations import (
        ListTaxAssociationsRequest,
        ListTaxAssociationsRequestTypedDict,
    )
    from .pausesubscription import (
        PauseSubscriptionRequest,
        PauseSubscriptionRequestTypedDict,
    )
    from .post_plans_id_clone import (
        PostPlansIDCloneRequest,
        PostPlansIDCloneRequestTypedDict,
    )
    from .previewsubscriptionchange import (
        PreviewSubscriptionChangeRequest,
        PreviewSubscriptionChangeRequestTypedDict,
    )
    from .processcreditnote import (
        ProcessCreditNoteRequest,
        ProcessCreditNoteRequestTypedDict,
    )
    from .processpayment import ProcessPaymentRequest, ProcessPaymentRequestTypedDict
    from .recalculateinvoice import (
        RecalculateInvoiceRequest,
        RecalculateInvoiceRequestTypedDict,
    )
    from .resumesubscription import (
        ResumeSubscriptionRequest,
        ResumeSubscriptionRequestTypedDict,
    )
    from .scheduleupdatebillingperiod import (
        ScheduleUpdateBillingPeriodRequest,
        ScheduleUpdateBillingPeriodRequestTypedDict,
        ScheduleUpdateBillingPeriodResponse,
        ScheduleUpdateBillingPeriodResponseTypedDict,
    )
    from .syncplanprices import SyncPlanPricesRequest, SyncPlanPricesRequestTypedDict
    from .terminatewallet import TerminateWalletRequest, TerminateWalletRequestTypedDict
    from .topupwallet import TopUpWalletRequest, TopUpWalletRequestTypedDict
    from .triggerinvoicecommswebhook import (
        TriggerInvoiceCommsWebhookRequest,
        TriggerInvoiceCommsWebhookRequestTypedDict,
    )
    from .triggerscheduledtaskrun import (
        TriggerScheduledTaskRunRequest,
        TriggerScheduledTaskRunRequestTypedDict,
    )
    from .updateaddon import UpdateAddonRequest, UpdateAddonRequestTypedDict
    from .updatecostsheet import UpdateCostsheetRequest, UpdateCostsheetRequestTypedDict
    from .updatecoupon import UpdateCouponRequest, UpdateCouponRequestTypedDict
    from .updatecreditgrant import (
        UpdateCreditGrantRequest,
        UpdateCreditGrantRequestTypedDict,
    )
    from .updatecustomer import UpdateCustomerRequest, UpdateCustomerRequestTypedDict
    from .updateentitlement import (
        UpdateEntitlementRequest,
        UpdateEntitlementRequestTypedDict,
    )
    from .updateenvironment import (
        UpdateEnvironmentRequest,
        UpdateEnvironmentRequestTypedDict,
    )
    from .updatefeature import UpdateFeatureRequest, UpdateFeatureRequestTypedDict
    from .updateinvoice import UpdateInvoiceRequest, UpdateInvoiceRequestTypedDict
    from .updateinvoicepaymentstatus import (
        UpdateInvoicePaymentStatusRequest,
        UpdateInvoicePaymentStatusRequestTypedDict,
    )
    from .updatepayment import UpdatePaymentRequest, UpdatePaymentRequestTypedDict
    from .updateplan import UpdatePlanRequest, UpdatePlanRequestTypedDict
    from .updateprice import UpdatePriceRequest, UpdatePriceRequestTypedDict
    from .updatepriceunit import UpdatePriceUnitRequest, UpdatePriceUnitRequestTypedDict
    from .updatescheduledtask import (
        UpdateScheduledTaskRequest,
        UpdateScheduledTaskRequestTypedDict,
    )
    from .updatesubscription import (
        UpdateSubscriptionRequest,
        UpdateSubscriptionRequestTypedDict,
    )
    from .updatesubscriptionlineitem import (
        UpdateSubscriptionLineItemRequest,
        UpdateSubscriptionLineItemRequestTypedDict,
    )
    from .updatetaskstatus import (
        UpdateTaskStatusRequest,
        UpdateTaskStatusRequestTypedDict,
    )
    from .updatetaxassociation import (
        UpdateTaxAssociationRequest,
        UpdateTaxAssociationRequestTypedDict,
    )
    from .updatetaxrate import UpdateTaxRateRequest, UpdateTaxRateRequestTypedDict
    from .updatewallet import UpdateWalletRequest, UpdateWalletRequestTypedDict
    from .voidcreditnote import VoidCreditNoteRequest, VoidCreditNoteRequestTypedDict
    from .voidinvoice import VoidInvoiceRequest, VoidInvoiceRequestTypedDict

__all__ = [
    "ActivateSubscriptionRequest",
    "ActivateSubscriptionRequestTypedDict",
    "AttemptInvoicePaymentRequest",
    "AttemptInvoicePaymentRequestTypedDict",
    "CancelSubscriptionRequest",
    "CancelSubscriptionRequestTypedDict",
    "CancelSubscriptionScheduleRequest",
    "CancelSubscriptionScheduleRequestTypedDict",
    "CreateOrUpdateIntegrationRequest",
    "CreateOrUpdateIntegrationRequestTypedDict",
    "CreateSubscriptionLineItemRequest",
    "CreateSubscriptionLineItemRequestTypedDict",
    "DeleteAPIKeyRequest",
    "DeleteAPIKeyRequestTypedDict",
    "DeleteAddonRequest",
    "DeleteAddonRequestTypedDict",
    "DeleteCostsheetRequest",
    "DeleteCostsheetRequestTypedDict",
    "DeleteCouponRequest",
    "DeleteCouponRequestTypedDict",
    "DeleteCreditGrantRequest",
    "DeleteCreditGrantRequestTypedDict",
    "DeleteCustomerRequest",
    "DeleteCustomerRequestTypedDict",
    "DeleteEntitlementRequest",
    "DeleteEntitlementRequestTypedDict",
    "DeleteEntityIntegrationMappingRequest",
    "DeleteEntityIntegrationMappingRequestTypedDict",
    "DeleteFeatureRequest",
    "DeleteFeatureRequestTypedDict",
    "DeleteGroupRequest",
    "DeleteGroupRequestTypedDict",
    "DeleteIntegrationRequest",
    "DeleteIntegrationRequestTypedDict",
    "DeletePaymentRequest",
    "DeletePaymentRequestTypedDict",
    "DeletePlanRequest",
    "DeletePlanRequestTypedDict",
    "DeletePriceRequest",
    "DeletePriceRequestTypedDict",
    "DeletePriceUnitRequest",
    "DeletePriceUnitRequestTypedDict",
    "DeleteScheduledTaskRequest",
    "DeleteScheduledTaskRequestTypedDict",
    "DeleteSubscriptionLineItemRequest",
    "DeleteSubscriptionLineItemRequestTypedDict",
    "DeleteTaxAssociationRequest",
    "DeleteTaxAssociationRequestTypedDict",
    "DeleteTaxRateRequest",
    "DeleteTaxRateRequestTypedDict",
    "DownloadTaskExportRequest",
    "DownloadTaskExportRequestTypedDict",
    "EntityType",
    "ExecuteSubscriptionChangeRequest",
    "ExecuteSubscriptionChangeRequestTypedDict",
    "FinalizeInvoiceRequest",
    "FinalizeInvoiceRequestTypedDict",
    "GetAddonByLookupKeyRequest",
    "GetAddonByLookupKeyRequestTypedDict",
    "GetAddonEntitlementsRequest",
    "GetAddonEntitlementsRequestTypedDict",
    "GetAddonRequest",
    "GetAddonRequestTypedDict",
    "GetCostsheetRequest",
    "GetCostsheetRequestTypedDict",
    "GetCouponRequest",
    "GetCouponRequestTypedDict",
    "GetCreditGrantRequest",
    "GetCreditGrantRequestTypedDict",
    "GetCreditNoteRequest",
    "GetCreditNoteRequestTypedDict",
    "GetCustomerByExternalIDRequest",
    "GetCustomerByExternalIDRequestTypedDict",
    "GetCustomerEntitlementsRequest",
    "GetCustomerEntitlementsRequestTypedDict",
    "GetCustomerInvoiceSummaryRequest",
    "GetCustomerInvoiceSummaryRequestTypedDict",
    "GetCustomerRequest",
    "GetCustomerRequestTypedDict",
    "GetCustomerUpcomingGrantsRequest",
    "GetCustomerUpcomingGrantsRequestTypedDict",
    "GetCustomerUsageSummaryRequest",
    "GetCustomerUsageSummaryRequestTypedDict",
    "GetCustomerWalletsRequest",
    "GetCustomerWalletsRequestTypedDict",
    "GetEntitlementRequest",
    "GetEntitlementRequestTypedDict",
    "GetEventRequest",
    "GetEventRequestTypedDict",
    "GetGroupRequest",
    "GetGroupRequestTypedDict",
    "GetIntegrationRequest",
    "GetIntegrationRequestTypedDict",
    "GetInvoicePdfRequest",
    "GetInvoicePdfRequestTypedDict",
    "GetInvoiceRequest",
    "GetInvoiceRequestTypedDict",
    "GetPaymentRequest",
    "GetPaymentRequestTypedDict",
    "GetPlanCreditGrantsRequest",
    "GetPlanCreditGrantsRequestTypedDict",
    "GetPlanEntitlementsRequest",
    "GetPlanEntitlementsRequestTypedDict",
    "GetPlanRequest",
    "GetPlanRequestTypedDict",
    "GetPriceByLookupKeyRequest",
    "GetPriceByLookupKeyRequestTypedDict",
    "GetPriceRequest",
    "GetPriceRequestTypedDict",
    "GetPriceUnitByCodeRequest",
    "GetPriceUnitByCodeRequestTypedDict",
    "GetPriceUnitRequest",
    "GetPriceUnitRequestTypedDict",
    "GetRbacRoleRequest",
    "GetRbacRoleRequestTypedDict",
    "GetScheduledTaskRequest",
    "GetScheduledTaskRequestTypedDict",
    "GetSubscriptionAddonAssociationsRequest",
    "GetSubscriptionAddonAssociationsRequestTypedDict",
    "GetSubscriptionEntitlementsRequest",
    "GetSubscriptionEntitlementsRequestTypedDict",
    "GetSubscriptionRequest",
    "GetSubscriptionRequestTypedDict",
    "GetSubscriptionScheduleRequest",
    "GetSubscriptionScheduleRequestTypedDict",
    "GetSubscriptionUpcomingGrantsRequest",
    "GetSubscriptionUpcomingGrantsRequestTypedDict",
    "GetSubscriptionV2Request",
    "GetSubscriptionV2RequestTypedDict",
    "GetTaskRequest",
    "GetTaskRequestTypedDict",
    "GetTaskResultRequest",
    "GetTaskResultRequestTypedDict",
    "GetTaxAssociationRequest",
    "GetTaxAssociationRequestTypedDict",
    "GetTaxRateRequest",
    "GetTaxRateRequestTypedDict",
    "GetTaxRatesOrder",
    "GetTaxRatesRequest",
    "GetTaxRatesRequestTypedDict",
    "GetTaxRatesStatus",
    "GetTenantByIDRequest",
    "GetTenantByIDRequestTypedDict",
    "GetWalletBalanceRequest",
    "GetWalletBalanceRequestTypedDict",
    "GetWalletRequest",
    "GetWalletRequestTypedDict",
    "GetWalletTransactionsOrder",
    "GetWalletTransactionsRequest",
    "GetWalletTransactionsRequestTypedDict",
    "GetWalletTransactionsStatus",
    "GetWalletsByCustomerIDRequest",
    "GetWalletsByCustomerIDRequestTypedDict",
    "HandleChargebeeWebhookRequest",
    "HandleChargebeeWebhookRequestTypedDict",
    "HandleHubspotWebhookRequest",
    "HandleHubspotWebhookRequestTypedDict",
    "HandleMoyasarWebhookRequest",
    "HandleMoyasarWebhookRequestTypedDict",
    "HandleNomodWebhookRequest",
    "HandleNomodWebhookRequestTypedDict",
    "HandleQuickbooksWebhookRequest",
    "HandleQuickbooksWebhookRequestTypedDict",
    "HandleRazorpayWebhookRequest",
    "HandleRazorpayWebhookRequestTypedDict",
    "HandleStripeWebhookRequest",
    "HandleStripeWebhookRequestTypedDict",
    "ListAPIKeysRequest",
    "ListAPIKeysRequestTypedDict",
    "ListAllSubscriptionSchedulesRequest",
    "ListAllSubscriptionSchedulesRequestTypedDict",
    "ListPaymentsOrder",
    "ListPaymentsRequest",
    "ListPaymentsRequestTypedDict",
    "ListPaymentsStatus",
    "ListPriceUnitsOrder",
    "ListPriceUnitsRequest",
    "ListPriceUnitsRequestTypedDict",
    "ListScheduledTasksRequest",
    "ListScheduledTasksRequestTypedDict",
    "ListSubscriptionPausesRequest",
    "ListSubscriptionPausesRequestTypedDict",
    "ListSubscriptionSchedulesRequest",
    "ListSubscriptionSchedulesRequestTypedDict",
    "ListTasksOrder",
    "ListTasksRequest",
    "ListTasksRequestTypedDict",
    "ListTasksStatus",
    "ListTaxAssociationsRequest",
    "ListTaxAssociationsRequestTypedDict",
    "PauseSubscriptionRequest",
    "PauseSubscriptionRequestTypedDict",
    "PostPlansIDCloneRequest",
    "PostPlansIDCloneRequestTypedDict",
    "PreviewSubscriptionChangeRequest",
    "PreviewSubscriptionChangeRequestTypedDict",
    "ProcessCreditNoteRequest",
    "ProcessCreditNoteRequestTypedDict",
    "ProcessPaymentRequest",
    "ProcessPaymentRequestTypedDict",
    "RecalculateInvoiceRequest",
    "RecalculateInvoiceRequestTypedDict",
    "ResumeSubscriptionRequest",
    "ResumeSubscriptionRequestTypedDict",
    "ScheduleUpdateBillingPeriodRequest",
    "ScheduleUpdateBillingPeriodRequestTypedDict",
    "ScheduleUpdateBillingPeriodResponse",
    "ScheduleUpdateBillingPeriodResponseTypedDict",
    "Scope",
    "SyncPlanPricesRequest",
    "SyncPlanPricesRequestTypedDict",
    "TaskStatus",
    "TaskType",
    "TerminateWalletRequest",
    "TerminateWalletRequestTypedDict",
    "TopUpWalletRequest",
    "TopUpWalletRequestTypedDict",
    "TransactionReason",
    "TransactionStatus",
    "TriggerInvoiceCommsWebhookRequest",
    "TriggerInvoiceCommsWebhookRequestTypedDict",
    "TriggerScheduledTaskRunRequest",
    "TriggerScheduledTaskRunRequestTypedDict",
    "Type",
    "UpdateAddonRequest",
    "UpdateAddonRequestTypedDict",
    "UpdateCostsheetRequest",
    "UpdateCostsheetRequestTypedDict",
    "UpdateCouponRequest",
    "UpdateCouponRequestTypedDict",
    "UpdateCreditGrantRequest",
    "UpdateCreditGrantRequestTypedDict",
    "UpdateCustomerRequest",
    "UpdateCustomerRequestTypedDict",
    "UpdateEntitlementRequest",
    "UpdateEntitlementRequestTypedDict",
    "UpdateEnvironmentRequest",
    "UpdateEnvironmentRequestTypedDict",
    "UpdateFeatureRequest",
    "UpdateFeatureRequestTypedDict",
    "UpdateInvoicePaymentStatusRequest",
    "UpdateInvoicePaymentStatusRequestTypedDict",
    "UpdateInvoiceRequest",
    "UpdateInvoiceRequestTypedDict",
    "UpdatePaymentRequest",
    "UpdatePaymentRequestTypedDict",
    "UpdatePlanRequest",
    "UpdatePlanRequestTypedDict",
    "UpdatePriceRequest",
    "UpdatePriceRequestTypedDict",
    "UpdatePriceUnitRequest",
    "UpdatePriceUnitRequestTypedDict",
    "UpdateScheduledTaskRequest",
    "UpdateScheduledTaskRequestTypedDict",
    "UpdateSubscriptionLineItemRequest",
    "UpdateSubscriptionLineItemRequestTypedDict",
    "UpdateSubscriptionRequest",
    "UpdateSubscriptionRequestTypedDict",
    "UpdateTaskStatusRequest",
    "UpdateTaskStatusRequestTypedDict",
    "UpdateTaxAssociationRequest",
    "UpdateTaxAssociationRequestTypedDict",
    "UpdateTaxRateRequest",
    "UpdateTaxRateRequestTypedDict",
    "UpdateWalletRequest",
    "UpdateWalletRequestTypedDict",
    "VoidCreditNoteRequest",
    "VoidCreditNoteRequestTypedDict",
    "VoidInvoiceRequest",
    "VoidInvoiceRequestTypedDict",
]

_dynamic_imports: dict[str, str] = {
    "ActivateSubscriptionRequest": ".activatesubscription",
    "ActivateSubscriptionRequestTypedDict": ".activatesubscription",
    "AttemptInvoicePaymentRequest": ".attemptinvoicepayment",
    "AttemptInvoicePaymentRequestTypedDict": ".attemptinvoicepayment",
    "CancelSubscriptionRequest": ".cancelsubscription",
    "CancelSubscriptionRequestTypedDict": ".cancelsubscription",
    "CancelSubscriptionScheduleRequest": ".cancelsubscriptionschedule",
    "CancelSubscriptionScheduleRequestTypedDict": ".cancelsubscriptionschedule",
    "CreateOrUpdateIntegrationRequest": ".createorupdateintegration",
    "CreateOrUpdateIntegrationRequestTypedDict": ".createorupdateintegration",
    "CreateSubscriptionLineItemRequest": ".createsubscriptionlineitem",
    "CreateSubscriptionLineItemRequestTypedDict": ".createsubscriptionlineitem",
    "DeleteAddonRequest": ".deleteaddon",
    "DeleteAddonRequestTypedDict": ".deleteaddon",
    "DeleteAPIKeyRequest": ".deleteapikey",
    "DeleteAPIKeyRequestTypedDict": ".deleteapikey",
    "DeleteCostsheetRequest": ".deletecostsheet",
    "DeleteCostsheetRequestTypedDict": ".deletecostsheet",
    "DeleteCouponRequest": ".deletecoupon",
    "DeleteCouponRequestTypedDict": ".deletecoupon",
    "DeleteCreditGrantRequest": ".deletecreditgrant",
    "DeleteCreditGrantRequestTypedDict": ".deletecreditgrant",
    "DeleteCustomerRequest": ".deletecustomer",
    "DeleteCustomerRequestTypedDict": ".deletecustomer",
    "DeleteEntitlementRequest": ".deleteentitlement",
    "DeleteEntitlementRequestTypedDict": ".deleteentitlement",
    "DeleteEntityIntegrationMappingRequest": ".deleteentityintegrationmapping",
    "DeleteEntityIntegrationMappingRequestTypedDict": ".deleteentityintegrationmapping",
    "DeleteFeatureRequest": ".deletefeature",
    "DeleteFeatureRequestTypedDict": ".deletefeature",
    "DeleteGroupRequest": ".deletegroup",
    "DeleteGroupRequestTypedDict": ".deletegroup",
    "DeleteIntegrationRequest": ".deleteintegration",
    "DeleteIntegrationRequestTypedDict": ".deleteintegration",
    "DeletePaymentRequest": ".deletepayment",
    "DeletePaymentRequestTypedDict": ".deletepayment",
    "DeletePlanRequest": ".deleteplan",
    "DeletePlanRequestTypedDict": ".deleteplan",
    "DeletePriceRequest": ".deleteprice",
    "DeletePriceRequestTypedDict": ".deleteprice",
    "DeletePriceUnitRequest": ".deletepriceunit",
    "DeletePriceUnitRequestTypedDict": ".deletepriceunit",
    "DeleteScheduledTaskRequest": ".deletescheduledtask",
    "DeleteScheduledTaskRequestTypedDict": ".deletescheduledtask",
    "DeleteSubscriptionLineItemRequest": ".deletesubscriptionlineitem",
    "DeleteSubscriptionLineItemRequestTypedDict": ".deletesubscriptionlineitem",
    "DeleteTaxAssociationRequest": ".deletetaxassociation",
    "DeleteTaxAssociationRequestTypedDict": ".deletetaxassociation",
    "DeleteTaxRateRequest": ".deletetaxrate",
    "DeleteTaxRateRequestTypedDict": ".deletetaxrate",
    "DownloadTaskExportRequest": ".downloadtaskexport",
    "DownloadTaskExportRequestTypedDict": ".downloadtaskexport",
    "ExecuteSubscriptionChangeRequest": ".executesubscriptionchange",
    "ExecuteSubscriptionChangeRequestTypedDict": ".executesubscriptionchange",
    "FinalizeInvoiceRequest": ".finalizeinvoice",
    "FinalizeInvoiceRequestTypedDict": ".finalizeinvoice",
    "GetAddonRequest": ".getaddon",
    "GetAddonRequestTypedDict": ".getaddon",
    "GetAddonByLookupKeyRequest": ".getaddonbylookupkey",
    "GetAddonByLookupKeyRequestTypedDict": ".getaddonbylookupkey",
    "GetAddonEntitlementsRequest": ".getaddonentitlements",
    "GetAddonEntitlementsRequestTypedDict": ".getaddonentitlements",
    "GetCostsheetRequest": ".getcostsheet",
    "GetCostsheetRequestTypedDict": ".getcostsheet",
    "GetCouponRequest": ".getcoupon",
    "GetCouponRequestTypedDict": ".getcoupon",
    "GetCreditGrantRequest": ".getcreditgrant",
    "GetCreditGrantRequestTypedDict": ".getcreditgrant",
    "GetCreditNoteRequest": ".getcreditnote",
    "GetCreditNoteRequestTypedDict": ".getcreditnote",
    "GetCustomerRequest": ".getcustomer",
    "GetCustomerRequestTypedDict": ".getcustomer",
    "GetCustomerByExternalIDRequest": ".getcustomerbyexternalid",
    "GetCustomerByExternalIDRequestTypedDict": ".getcustomerbyexternalid",
    "GetCustomerEntitlementsRequest": ".getcustomerentitlements",
    "GetCustomerEntitlementsRequestTypedDict": ".getcustomerentitlements",
    "GetCustomerInvoiceSummaryRequest": ".getcustomerinvoicesummary",
    "GetCustomerInvoiceSummaryRequestTypedDict": ".getcustomerinvoicesummary",
    "GetCustomerUpcomingGrantsRequest": ".getcustomerupcominggrants",
    "GetCustomerUpcomingGrantsRequestTypedDict": ".getcustomerupcominggrants",
    "GetCustomerUsageSummaryRequest": ".getcustomerusagesummary",
    "GetCustomerUsageSummaryRequestTypedDict": ".getcustomerusagesummary",
    "GetCustomerWalletsRequest": ".getcustomerwallets",
    "GetCustomerWalletsRequestTypedDict": ".getcustomerwallets",
    "GetEntitlementRequest": ".getentitlement",
    "GetEntitlementRequestTypedDict": ".getentitlement",
    "GetEventRequest": ".getevent",
    "GetEventRequestTypedDict": ".getevent",
    "GetGroupRequest": ".getgroup",
    "GetGroupRequestTypedDict": ".getgroup",
    "GetIntegrationRequest": ".getintegration",
    "GetIntegrationRequestTypedDict": ".getintegration",
    "GetInvoiceRequest": ".getinvoice",
    "GetInvoiceRequestTypedDict": ".getinvoice",
    "GetInvoicePdfRequest": ".getinvoicepdf",
    "GetInvoicePdfRequestTypedDict": ".getinvoicepdf",
    "GetPaymentRequest": ".getpayment",
    "GetPaymentRequestTypedDict": ".getpayment",
    "GetPlanRequest": ".getplan",
    "GetPlanRequestTypedDict": ".getplan",
    "GetPlanCreditGrantsRequest": ".getplancreditgrants",
    "GetPlanCreditGrantsRequestTypedDict": ".getplancreditgrants",
    "GetPlanEntitlementsRequest": ".getplanentitlements",
    "GetPlanEntitlementsRequestTypedDict": ".getplanentitlements",
    "GetPriceRequest": ".getprice",
    "GetPriceRequestTypedDict": ".getprice",
    "GetPriceByLookupKeyRequest": ".getpricebylookupkey",
    "GetPriceByLookupKeyRequestTypedDict": ".getpricebylookupkey",
    "GetPriceUnitRequest": ".getpriceunit",
    "GetPriceUnitRequestTypedDict": ".getpriceunit",
    "GetPriceUnitByCodeRequest": ".getpriceunitbycode",
    "GetPriceUnitByCodeRequestTypedDict": ".getpriceunitbycode",
    "GetRbacRoleRequest": ".getrbacrole",
    "GetRbacRoleRequestTypedDict": ".getrbacrole",
    "GetScheduledTaskRequest": ".getscheduledtask",
    "GetScheduledTaskRequestTypedDict": ".getscheduledtask",
    "GetSubscriptionRequest": ".getsubscription",
    "GetSubscriptionRequestTypedDict": ".getsubscription",
    "GetSubscriptionAddonAssociationsRequest": ".getsubscriptionaddonassociations",
    "GetSubscriptionAddonAssociationsRequestTypedDict": ".getsubscriptionaddonassociations",
    "GetSubscriptionEntitlementsRequest": ".getsubscriptionentitlements",
    "GetSubscriptionEntitlementsRequestTypedDict": ".getsubscriptionentitlements",
    "GetSubscriptionScheduleRequest": ".getsubscriptionschedule",
    "GetSubscriptionScheduleRequestTypedDict": ".getsubscriptionschedule",
    "GetSubscriptionUpcomingGrantsRequest": ".getsubscriptionupcominggrants",
    "GetSubscriptionUpcomingGrantsRequestTypedDict": ".getsubscriptionupcominggrants",
    "GetSubscriptionV2Request": ".getsubscriptionv2",
    "GetSubscriptionV2RequestTypedDict": ".getsubscriptionv2",
    "GetTaskRequest": ".gettask",
    "GetTaskRequestTypedDict": ".gettask",
    "GetTaskResultRequest": ".gettaskresult",
    "GetTaskResultRequestTypedDict": ".gettaskresult",
    "GetTaxAssociationRequest": ".gettaxassociation",
    "GetTaxAssociationRequestTypedDict": ".gettaxassociation",
    "GetTaxRateRequest": ".gettaxrate",
    "GetTaxRateRequestTypedDict": ".gettaxrate",
    "GetTaxRatesOrder": ".gettaxrates",
    "GetTaxRatesRequest": ".gettaxrates",
    "GetTaxRatesRequestTypedDict": ".gettaxrates",
    "GetTaxRatesStatus": ".gettaxrates",
    "Scope": ".gettaxrates",
    "GetTenantByIDRequest": ".gettenantbyid",
    "GetTenantByIDRequestTypedDict": ".gettenantbyid",
    "GetWalletRequest": ".getwallet",
    "GetWalletRequestTypedDict": ".getwallet",
    "GetWalletBalanceRequest": ".getwalletbalance",
    "GetWalletBalanceRequestTypedDict": ".getwalletbalance",
    "GetWalletsByCustomerIDRequest": ".getwalletsbycustomerid",
    "GetWalletsByCustomerIDRequestTypedDict": ".getwalletsbycustomerid",
    "GetWalletTransactionsOrder": ".getwallettransactions",
    "GetWalletTransactionsRequest": ".getwallettransactions",
    "GetWalletTransactionsRequestTypedDict": ".getwallettransactions",
    "GetWalletTransactionsStatus": ".getwallettransactions",
    "TransactionReason": ".getwallettransactions",
    "TransactionStatus": ".getwallettransactions",
    "Type": ".getwallettransactions",
    "HandleChargebeeWebhookRequest": ".handlechargebeewebhook",
    "HandleChargebeeWebhookRequestTypedDict": ".handlechargebeewebhook",
    "HandleHubspotWebhookRequest": ".handlehubspotwebhook",
    "HandleHubspotWebhookRequestTypedDict": ".handlehubspotwebhook",
    "HandleMoyasarWebhookRequest": ".handlemoyasarwebhook",
    "HandleMoyasarWebhookRequestTypedDict": ".handlemoyasarwebhook",
    "HandleNomodWebhookRequest": ".handlenomodwebhook",
    "HandleNomodWebhookRequestTypedDict": ".handlenomodwebhook",
    "HandleQuickbooksWebhookRequest": ".handlequickbookswebhook",
    "HandleQuickbooksWebhookRequestTypedDict": ".handlequickbookswebhook",
    "HandleRazorpayWebhookRequest": ".handlerazorpaywebhook",
    "HandleRazorpayWebhookRequestTypedDict": ".handlerazorpaywebhook",
    "HandleStripeWebhookRequest": ".handlestripewebhook",
    "HandleStripeWebhookRequestTypedDict": ".handlestripewebhook",
    "ListAllSubscriptionSchedulesRequest": ".listallsubscriptionschedules",
    "ListAllSubscriptionSchedulesRequestTypedDict": ".listallsubscriptionschedules",
    "ListAPIKeysRequest": ".listapikeys",
    "ListAPIKeysRequestTypedDict": ".listapikeys",
    "ListPaymentsOrder": ".listpayments",
    "ListPaymentsRequest": ".listpayments",
    "ListPaymentsRequestTypedDict": ".listpayments",
    "ListPaymentsStatus": ".listpayments",
    "ListPriceUnitsOrder": ".listpriceunits",
    "ListPriceUnitsRequest": ".listpriceunits",
    "ListPriceUnitsRequestTypedDict": ".listpriceunits",
    "ListScheduledTasksRequest": ".listscheduledtasks",
    "ListScheduledTasksRequestTypedDict": ".listscheduledtasks",
    "ListSubscriptionPausesRequest": ".listsubscriptionpauses",
    "ListSubscriptionPausesRequestTypedDict": ".listsubscriptionpauses",
    "ListSubscriptionSchedulesRequest": ".listsubscriptionschedules",
    "ListSubscriptionSchedulesRequestTypedDict": ".listsubscriptionschedules",
    "EntityType": ".listtasks",
    "ListTasksOrder": ".listtasks",
    "ListTasksRequest": ".listtasks",
    "ListTasksRequestTypedDict": ".listtasks",
    "ListTasksStatus": ".listtasks",
    "TaskStatus": ".listtasks",
    "TaskType": ".listtasks",
    "ListTaxAssociationsRequest": ".listtaxassociations",
    "ListTaxAssociationsRequestTypedDict": ".listtaxassociations",
    "PauseSubscriptionRequest": ".pausesubscription",
    "PauseSubscriptionRequestTypedDict": ".pausesubscription",
    "PostPlansIDCloneRequest": ".post_plans_id_clone",
    "PostPlansIDCloneRequestTypedDict": ".post_plans_id_clone",
    "PreviewSubscriptionChangeRequest": ".previewsubscriptionchange",
    "PreviewSubscriptionChangeRequestTypedDict": ".previewsubscriptionchange",
    "ProcessCreditNoteRequest": ".processcreditnote",
    "ProcessCreditNoteRequestTypedDict": ".processcreditnote",
    "ProcessPaymentRequest": ".processpayment",
    "ProcessPaymentRequestTypedDict": ".processpayment",
    "RecalculateInvoiceRequest": ".recalculateinvoice",
    "RecalculateInvoiceRequestTypedDict": ".recalculateinvoice",
    "ResumeSubscriptionRequest": ".resumesubscription",
    "ResumeSubscriptionRequestTypedDict": ".resumesubscription",
    "ScheduleUpdateBillingPeriodRequest": ".scheduleupdatebillingperiod",
    "ScheduleUpdateBillingPeriodRequestTypedDict": ".scheduleupdatebillingperiod",
    "ScheduleUpdateBillingPeriodResponse": ".scheduleupdatebillingperiod",
    "ScheduleUpdateBillingPeriodResponseTypedDict": ".scheduleupdatebillingperiod",
    "SyncPlanPricesRequest": ".syncplanprices",
    "SyncPlanPricesRequestTypedDict": ".syncplanprices",
    "TerminateWalletRequest": ".terminatewallet",
    "TerminateWalletRequestTypedDict": ".terminatewallet",
    "TopUpWalletRequest": ".topupwallet",
    "TopUpWalletRequestTypedDict": ".topupwallet",
    "TriggerInvoiceCommsWebhookRequest": ".triggerinvoicecommswebhook",
    "TriggerInvoiceCommsWebhookRequestTypedDict": ".triggerinvoicecommswebhook",
    "TriggerScheduledTaskRunRequest": ".triggerscheduledtaskrun",
    "TriggerScheduledTaskRunRequestTypedDict": ".triggerscheduledtaskrun",
    "UpdateAddonRequest": ".updateaddon",
    "UpdateAddonRequestTypedDict": ".updateaddon",
    "UpdateCostsheetRequest": ".updatecostsheet",
    "UpdateCostsheetRequestTypedDict": ".updatecostsheet",
    "UpdateCouponRequest": ".updatecoupon",
    "UpdateCouponRequestTypedDict": ".updatecoupon",
    "UpdateCreditGrantRequest": ".updatecreditgrant",
    "UpdateCreditGrantRequestTypedDict": ".updatecreditgrant",
    "UpdateCustomerRequest": ".updatecustomer",
    "UpdateCustomerRequestTypedDict": ".updatecustomer",
    "UpdateEntitlementRequest": ".updateentitlement",
    "UpdateEntitlementRequestTypedDict": ".updateentitlement",
    "UpdateEnvironmentRequest": ".updateenvironment",
    "UpdateEnvironmentRequestTypedDict": ".updateenvironment",
    "UpdateFeatureRequest": ".updatefeature",
    "UpdateFeatureRequestTypedDict": ".updatefeature",
    "UpdateInvoiceRequest": ".updateinvoice",
    "UpdateInvoiceRequestTypedDict": ".updateinvoice",
    "UpdateInvoicePaymentStatusRequest": ".updateinvoicepaymentstatus",
    "UpdateInvoicePaymentStatusRequestTypedDict": ".updateinvoicepaymentstatus",
    "UpdatePaymentRequest": ".updatepayment",
    "UpdatePaymentRequestTypedDict": ".updatepayment",
    "UpdatePlanRequest": ".updateplan",
    "UpdatePlanRequestTypedDict": ".updateplan",
    "UpdatePriceRequest": ".updateprice",
    "UpdatePriceRequestTypedDict": ".updateprice",
    "UpdatePriceUnitRequest": ".updatepriceunit",
    "UpdatePriceUnitRequestTypedDict": ".updatepriceunit",
    "UpdateScheduledTaskRequest": ".updatescheduledtask",
    "UpdateScheduledTaskRequestTypedDict": ".updatescheduledtask",
    "UpdateSubscriptionRequest": ".updatesubscription",
    "UpdateSubscriptionRequestTypedDict": ".updatesubscription",
    "UpdateSubscriptionLineItemRequest": ".updatesubscriptionlineitem",
    "UpdateSubscriptionLineItemRequestTypedDict": ".updatesubscriptionlineitem",
    "UpdateTaskStatusRequest": ".updatetaskstatus",
    "UpdateTaskStatusRequestTypedDict": ".updatetaskstatus",
    "UpdateTaxAssociationRequest": ".updatetaxassociation",
    "UpdateTaxAssociationRequestTypedDict": ".updatetaxassociation",
    "UpdateTaxRateRequest": ".updatetaxrate",
    "UpdateTaxRateRequestTypedDict": ".updatetaxrate",
    "UpdateWalletRequest": ".updatewallet",
    "UpdateWalletRequestTypedDict": ".updatewallet",
    "VoidCreditNoteRequest": ".voidcreditnote",
    "VoidCreditNoteRequestTypedDict": ".voidcreditnote",
    "VoidInvoiceRequest": ".voidinvoice",
    "VoidInvoiceRequestTypedDict": ".voidinvoice",
}


def __getattr__(attr_name: str) -> Any:
    return lazy_getattr(
        attr_name, package=__package__, dynamic_imports=_dynamic_imports
    )


def __dir__():
    return lazy_dir(dynamic_imports=_dynamic_imports)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .windowsize import WindowSize
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoGetUsageAnalyticsRequestTypedDict(TypedDict):
    external_customer_id: str
    end_time: NotRequired[str]
    expand: NotRequired[List[str]]
    r"""allowed values: \"price\", \"meter\", \"feature\", \"subscription_line_item\",\"plan\",\"addon\" """
    feature_ids: NotRequired[List[str]]
    group_by: NotRequired[List[str]]
    r"""allowed values: \"source\", \"feature_id\", \"properties.<field_name>\" """
    property_filters: NotRequired[Dict[str, List[str]]]
    r"""Property filters to filter the events by the keys in `properties` field of the event"""
    sources: NotRequired[List[str]]
    start_time: NotRequired[str]
    window_size: NotRequired[WindowSize]


class DtoGetUsageAnalyticsRequest(BaseModel):
    external_customer_id: str

    end_time: Optional[str] = None

    expand: Optional[List[str]] = None
    r"""allowed values: \"price\", \"meter\", \"feature\", \"subscription_line_item\",\"plan\",\"addon\" """

    feature_ids: Optional[List[str]] = None

    group_by: Optional[List[str]] = None
    r"""allowed values: \"source\", \"feature_id\", \"properties.<field_name>\" """

    property_filters: Optional[Dict[str, List[str]]] = None
    r"""Property filters to filter the events by the keys in `properties` field of the event"""

    sources: Optional[List[str]] = None

    start_time: Optional[str] = None

    window_size: Optional[WindowSize] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "end_time",
                "expand",
                "feature_ids",
                "group_by",
                "property_filters",
                "sources",
                "start_time",
                "window_size",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .alertsettings import AlertSettings, AlertSettingsTypedDict
from .autotopup import AutoTopup, AutoTopupTypedDict
from .wallettype import WalletType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateWalletRequestTypedDict(TypedDict):
    currency: str
    alert_settings: NotRequired[AlertSettingsTypedDict]
    auto_topup: NotRequired[AutoTopupTypedDict]
    conversion_rate: NotRequired[str]
    r"""amount in the currency =  number of credits * conversion_rate
    ex if conversion_rate is 1, then 1 USD = 1 credit
    ex if conversion_rate is 2, then 1 USD = 0.5 credits
    ex if conversion_rate is 0.5, then 1 USD = 2 credits
    """
    customer_id: NotRequired[str]
    description: NotRequired[str]
    external_customer_id: NotRequired[str]
    r"""external_customer_id is the customer id in the external system"""
    initial_credits_expiry_date_utc: NotRequired[str]
    r"""initial_credits_expiry_date_utc is the expiry date in UTC timezone (optional to set nil means no expiry)
    ex 2025-01-01 00:00:00 UTC
    """
    initial_credits_to_load: NotRequired[str]
    r"""initial_credits_to_load is the number of credits to load to the wallet
    if not provided, the wallet will be created with 0 balance
    NOTE: this is not the amount in the currency, but the number of credits
    """
    initial_credits_to_load_expiry_date: NotRequired[int]
    r"""initial_credits_to_load_expiry_date YYYYMMDD format in UTC timezone (optional to set nil means no expiry)
    for ex 20250101 means the credits will expire on 2025-01-01 00:00:00 UTC
    hence they will be available for use until 2024-12-31 23:59:59 UTC
    """
    metadata: NotRequired[Dict[str, str]]
    price_unit: NotRequired[str]
    r"""price_unit is the code of the price unit to use for wallet creation
    If provided, the price unit will be used to set the currency and conversion rate of the wallet:
    - currency: set to price unit's base_currency
    - conversion_rate: set to price unit's conversion_rate
    """
    topup_conversion_rate: NotRequired[str]
    r"""topup_conversion_rate is the conversion rate for the topup to the currency
    ex if topup_conversion_rate is 1, then 1 USD = 1 credit
    ex if topup_conversion_rate is 2, then 1 USD = 0.5 credits
    ex if topup_conversion_rate is 0.5, then 1 USD = 2 credits
    """
    wallet_type: NotRequired[WalletType]


class DtoCreateWalletRequest(BaseModel):
    currency: str

    alert_settings: Optional[AlertSettings] = None

    auto_topup: Optional[AutoTopup] = None

    conversion_rate: Optional[str] = "1"
    r"""amount in the currency =  number of credits * conversion_rate
    ex if conversion_rate is 1, then 1 USD = 1 credit
    ex if conversion_rate is 2, then 1 USD = 0.5 credits
    ex if conversion_rate is 0.5, then 1 USD = 2 credits
    """

    customer_id: Optional[str] = None

    description: Optional[str] = None

    external_customer_id: Optional[str] = None
    r"""external_customer_id is the customer id in the external system"""

    initial_credits_expiry_date_utc: Optional[str] = None
    r"""initial_credits_expiry_date_utc is the expiry date in UTC timezone (optional to set nil means no expiry)
    ex 2025-01-01 00:00:00 UTC
    """

    initial_credits_to_load: Optional[str] = "0"
    r"""initial_credits_to_load is the number of credits to load to the wallet
    if not provided, the wallet will be created with 0 balance
    NOTE: this is not the amount in the currency, but the number of credits
    """

    initial_credits_to_load_expiry_date: Optional[int] = None
    r"""initial_credits_to_load_expiry_date YYYYMMDD format in UTC timezone (optional to set nil means no expiry)
    for ex 20250101 means the credits will expire on 2025-01-01 00:00:00 UTC
    hence they will be available for use until 2024-12-31 23:59:59 UTC
    """

    metadata: Optional[Dict[str, str]] = None

    price_unit: Optional[str] = None
    r"""price_unit is the code of the price unit to use for wallet creation
    If provided, the price unit will be used to set the currency and conversion rate of the wallet:
    - currency: set to price unit's base_currency
    - conversion_rate: set to price unit's conversion_rate
    """

    topup_conversion_rate: Optional[str] = None
    r"""topup_conversion_rate is the conversion rate for the topup to the currency
    ex if topup_conversion_rate is 1, then 1 USD = 1 credit
    ex if topup_conversion_rate is 2, then 1 USD = 0.5 credits
    ex if topup_conversion_rate is 0.5, then 1 USD = 2 credits
    """

    wallet_type: Optional[WalletType] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "alert_settings",
                "auto_topup",
                "conversion_rate",
                "customer_id",
                "description",
                "external_customer_id",
                "initial_credits_expiry_date_utc",
                "initial_credits_to_load",
                "initial_credits_to_load_expiry_date",
                "metadata",
                "price_unit",
                "topup_conversion_rate",
                "wallet_type",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoCustomerInvoiceSummaryTypedDict(TypedDict):
    currency: NotRequired[str]
    r"""currency is the three-letter ISO currency code for this summary"""
    customer_id: NotRequired[str]
    r"""customer_id is the unique identifier of the customer"""
    overdue_invoice_count: NotRequired[int]
    r"""overdue_invoice_count is the number of overdue invoices for this customer in this currency"""
    total_invoice_count: NotRequired[int]
    r"""total_invoice_count is the total number of invoices for this customer in this currency"""
    total_overdue_amount: NotRequired[str]
    r"""total_overdue_amount is the total amount of overdue invoices in this currency"""
    total_revenue_amount: NotRequired[str]
    r"""total_revenue_amount is the total revenue generated from this customer in this currency"""
    total_unpaid_amount: NotRequired[str]
    r"""total_unpaid_amount is the total amount of unpaid invoices in this currency"""
    unpaid_fixed_charges: NotRequired[str]
    r"""unpaid_fixed_charges is the total amount of unpaid fixed charges in this currency"""
    unpaid_invoice_count: NotRequired[int]
    r"""unpaid_invoice_count is the number of unpaid invoices for this customer in this currency"""
    unpaid_usage_charges: NotRequired[str]
    r"""unpaid_usage_charges is the total amount of unpaid usage-based charges in this currency"""


class DtoCustomerInvoiceSummary(BaseModel):
    currency: Optional[str] = None
    r"""currency is the three-letter ISO currency code for this summary"""

    customer_id: Optional[str] = None
    r"""customer_id is the unique identifier of the customer"""

    overdue_invoice_count: Optional[int] = None
    r"""overdue_invoice_count is the number of overdue invoices for this customer in this currency"""

    total_invoice_count: Optional[int] = None
    r"""total_invoice_count is the total number of invoices for this customer in this currency"""

    total_overdue_amount: Optional[str] = None
    r"""total_overdue_amount is the total amount of overdue invoices in this currency"""

    total_revenue_amount: Optional[str] = None
    r"""total_revenue_amount is the total revenue generated from this customer in this currency"""

    total_unpaid_amount: Optional[str] = None
    r"""total_unpaid_amount is the total amount of unpaid invoices in this currency"""

    unpaid_fixed_charges: Optional[str] = None
    r"""unpaid_fixed_charges is the total amount of unpaid fixed charges in this currency"""

    unpaid_invoice_count: Optional[int] = None
    r"""unpaid_invoice_count is the number of unpaid invoices for this customer in this currency"""

    unpaid_usage_charges: Optional[str] = None
    r"""unpaid_usage_charges is the total amount of unpaid usage-based charges in this currency"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "currency",
                "customer_id",
                "overdue_invoice_count",
                "total_invoice_count",
                "total_overdue_amount",
                "total_revenue_amount",
                "total_unpaid_amount",
                "unpaid_fixed_charges",
                "unpaid_invoice_count",
                "unpaid_usage_charges",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .windowsize import WindowSize
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoGetUsageByMeterRequestTypedDict(TypedDict):
    meter_id: str
    billing_anchor: NotRequired[str]
    r"""BillingAnchor enables custom monthly billing periods for meter usage aggregation.

    Usage guidelines:
    - Only effective when WindowSize = \"MONTH\" 
    - For other window sizes (DAY, HOUR, WEEK), this field is ignored
    - When nil, uses standard calendar months (1st to 1st)
    - When provided, creates custom monthly periods (e.g., 5th to 5th)

    Common use cases:
    - Subscription billing periods that don't align with calendar months
    - Customer-specific billing cycles (e.g., signed up on 15th)
    - Multi-tenant systems with different billing anchor dates

    Example: If BillingAnchor = \"2024-03-05T14:30:45Z\" and WindowSize = \"MONTH\":
    - March period: 2024-03-05 14:30:45 to 2024-04-05 14:30:45
    - April period: 2024-04-05 14:30:45 to 2024-05-05 14:30:45
    """
    bucket_size: NotRequired[WindowSize]
    customer_id: NotRequired[str]
    end_time: NotRequired[str]
    external_customer_id: NotRequired[str]
    filters: NotRequired[Dict[str, List[str]]]
    start_time: NotRequired[str]
    window_size: NotRequired[WindowSize]


class DtoGetUsageByMeterRequest(BaseModel):
    meter_id: str

    billing_anchor: Optional[str] = None
    r"""BillingAnchor enables custom monthly billing periods for meter usage aggregation.

    Usage guidelines:
    - Only effective when WindowSize = \"MONTH\" 
    - For other window sizes (DAY, HOUR, WEEK), this field is ignored
    - When nil, uses standard calendar months (1st to 1st)
    - When provided, creates custom monthly periods (e.g., 5th to 5th)

    Common use cases:
    - Subscription billing periods that don't align with calendar months
    - Customer-specific billing cycles (e.g., signed up on 15th)
    - Multi-tenant systems with different billing anchor dates

    Example: If BillingAnchor = \"2024-03-05T14:30:45Z\" and WindowSize = \"MONTH\":
    - March period: 2024-03-05 14:30:45 to 2024-04-05 14:30:45
    - April period: 2024-04-05 14:30:45 to 2024-05-05 14:30:45
    """

    bucket_size: Optional[WindowSize] = None

    customer_id: Optional[str] = None

    end_time: Optional[str] = None

    external_customer_id: Optional[str] = None

    filters: Optional[Dict[str, List[str]]] = None

    start_time: Optional[str] = None

    window_size: Optional[WindowSize] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "billing_anchor",
                "bucket_size",
                "customer_id",
                "end_time",
                "external_customer_id",
                "filters",
                "start_time",
                "window_size",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .secretprovider import SecretProvider
from .secrettype import SecretType
from .status import Status
from .usertype import UserType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoSecretResponseTypedDict(TypedDict):
    created_at: NotRequired[str]
    display_id: NotRequired[str]
    expires_at: NotRequired[str]
    id: NotRequired[str]
    last_used_at: NotRequired[str]
    name: NotRequired[str]
    provider: NotRequired[SecretProvider]
    roles: NotRequired[List[str]]
    r"""RBAC roles"""
    status: NotRequired[Status]
    type: NotRequired[SecretType]
    updated_at: NotRequired[str]
    user_type: NotRequired[UserType]


class DtoSecretResponse(BaseModel):
    created_at: Optional[str] = None

    display_id: Optional[str] = None

    expires_at: Optional[str] = None

    id: Optional[str] = None

    last_used_at: Optional[str] = None

    name: Optional[str] = None

    provider: Optional[SecretProvider] = None

    roles: Optional[List[str]] = None
    r"""RBAC roles"""

    status: Optional[Status] = None

    type: Optional[SecretType] = None

    updated_at: Optional[str] = None

    user_type: Optional[UserType] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "created_at",
                "display_id",
                "expires_at",
                "id",
                "last_used_at",
                "name",
                "provider",
                "roles",
                "status",
                "type",
                "updated_at",
                "user_type",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingperiod import BillingPeriod
from .commitmenttype import CommitmentType
from .dto_subscriptionpricecreaterequest import (
    DtoSubscriptionPriceCreateRequest,
    DtoSubscriptionPriceCreateRequestTypedDict,
)
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateSubscriptionLineItemRequestTypedDict(TypedDict):
    commitment_amount: NotRequired[float]
    r"""Commitment fields"""
    commitment_duration: NotRequired[BillingPeriod]
    commitment_overage_factor: NotRequired[float]
    commitment_quantity: NotRequired[float]
    commitment_true_up_enabled: NotRequired[bool]
    commitment_type: NotRequired[CommitmentType]
    commitment_windowed: NotRequired[bool]
    display_name: NotRequired[str]
    end_date: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    price: NotRequired[DtoSubscriptionPriceCreateRequestTypedDict]
    price_id: NotRequired[str]
    r"""PriceID references an existing price (plan, addon, or subscription-scoped). Exactly one of price_id or price must be set."""
    quantity: NotRequired[float]
    start_date: NotRequired[str]
    subscription_phase_id: NotRequired[str]


class DtoCreateSubscriptionLineItemRequest(BaseModel):
    commitment_amount: Optional[float] = None
    r"""Commitment fields"""

    commitment_duration: Optional[BillingPeriod] = None

    commitment_overage_factor: Optional[float] = None

    commitment_quantity: Optional[float] = None

    commitment_true_up_enabled: Optional[bool] = None

    commitment_type: Optional[CommitmentType] = None

    commitment_windowed: Optional[bool] = None

    display_name: Optional[str] = None

    end_date: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    price: Optional[DtoSubscriptionPriceCreateRequest] = None

    price_id: Optional[str] = None
    r"""PriceID references an existing price (plan, addon, or subscription-scoped). Exactly one of price_id or price must be set."""

    quantity: Optional[float] = None

    start_date: Optional[str] = None

    subscription_phase_id: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "commitment_amount",
                "commitment_duration",
                "commitment_overage_factor",
                "commitment_quantity",
                "commitment_true_up_enabled",
                "commitment_type",
                "commitment_windowed",
                "display_name",
                "end_date",
                "metadata",
                "price",
                "price_id",
                "quantity",
                "start_date",
                "subscription_phase_id",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import httpx
from typing import Optional
from dataclasses import dataclass

from flexprice_py.errors import FlexpriceError


@dataclass(unsafe_hash=True)
class ResponseValidationError(FlexpriceError):
    """Error raised when there is a type mismatch between the response data and the expected Pydantic model."""

    def __init__(
        self,
        message: str,
        raw_response: httpx.Response,
        cause: Exception,
        body: Optional[str] = None,
    ):
        message = f"{message}: {cause}"
        super().__init__(message, raw_response, body)

    @property
    def cause(self):
        """Normally the Pydantic ValidationError"""
        return self.__cause__



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel
from flexprice_py.utils import FieldMetadata, SecurityMetadata
from typing_extensions import Annotated, TypedDict


class SecurityTypedDict(TypedDict):
    api_key_auth: str


class Security(BaseModel):
    api_key_auth: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="apiKey",
                sub_type="header",
                field_name="x-api-key",
            )
        ),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_tenantresponse import DtoTenantResponse, DtoTenantResponseTypedDict
from .usertype import UserType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoUserResponseTypedDict(TypedDict):
    email: NotRequired[str]
    r"""Empty for service accounts"""
    id: NotRequired[str]
    roles: NotRequired[List[str]]
    tenant: NotRequired[DtoTenantResponseTypedDict]
    type: NotRequired[UserType]


class DtoUserResponse(BaseModel):
    email: Optional[str] = None
    r"""Empty for service accounts"""

    id: Optional[str] = None

    roles: Optional[List[str]] = None

    tenant: Optional[DtoTenantResponse] = None

    type: Optional[UserType] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["email", "id", "roles", "tenant", "type"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


PaymentStatus = Union[
    Literal[
        "INITIATED",
        "PENDING",
        "PROCESSING",
        "SUCCEEDED",
        "OVERPAID",
        "FAILED",
        "REFUNDED",
        "PARTIALLY_REFUNDED",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoEventTypedDict(TypedDict):
    customer_id: NotRequired[str]
    environment_id: NotRequired[str]
    event_name: NotRequired[str]
    external_customer_id: NotRequired[str]
    id: NotRequired[str]
    properties: NotRequired[Dict[str, Any]]
    source: NotRequired[str]
    timestamp: NotRequired[str]


class DtoEvent(BaseModel):
    customer_id: Optional[str] = None

    environment_id: Optional[str] = None

    event_name: Optional[str] = None

    external_customer_id: Optional[str] = None

    id: Optional[str] = None

    properties: Optional[Dict[str, Any]] = None

    source: Optional[str] = None

    timestamp: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "customer_id",
                "environment_id",
                "event_name",
                "external_customer_id",
                "id",
                "properties",
                "source",
                "timestamp",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoIngestEventRequestTypedDict(TypedDict):
    event_name: str
    external_customer_id: str
    customer_id: NotRequired[str]
    event_id: NotRequired[str]
    properties: NotRequired[Dict[str, str]]
    r"""Handled separately for dynamic columns"""
    source: NotRequired[str]
    timestamp: NotRequired[str]
    r"""Handled separately due to parsing"""


class DtoIngestEventRequest(BaseModel):
    event_name: str

    external_customer_id: str

    customer_id: Optional[str] = None

    event_id: Optional[str] = None

    properties: Optional[Dict[str, str]] = None
    r"""Handled separately for dynamic columns"""

    source: Optional[str] = None

    timestamp: Optional[str] = None
    r"""Handled separately due to parsing"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["customer_id", "event_id", "properties", "source", "timestamp"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
